@@ -1,4 +1,7 @@
-/* $Id: acconfig.h,v 1.69 1997/05/25 07:36:26 assar Exp $ */
+/* $Id: acconfig.h,v 1.79 1997/11/09 13:55:55 joda Exp $ */
+
+/* Define this to the type ssize_t should be */
+#undef ssize_t
 
 /*  Define this if RETSIGTYPE == void  */
 #undef VOID_RETSIGTYPE
@@ -24,6 +27,9 @@
 /*  Define this if struct utmpx have ut_syslen  */
 #undef HAVE_UT_SYSLEN
 
+/*  Define this if struct utmpx have ut_syslen  */
+#undef HAVE_UT_EXIT
+
 /*  Define this if struct winsize is declared in sys/termios.h */
 #undef HAVE_STRUCT_WINSIZE
 
@@ -61,9 +67,15 @@
 /* define if the system is missing a prototype for crypt() */
 #undef NEED_CRYPT_PROTO
 
+/* define if the system is missing a prototype for fclose() */
+#undef NEED_FCLOSE_PROTO
+
 /* define if the system is missing a prototype for strtok_r() */
 #undef NEED_STRTOK_R_PROTO
 
+/* define if the system is missing a prototype for getusershell() */
+#undef NEED_GETUSERSHELL_PROTO
+
 /* define if /bin/ls takes -A */
 #undef HAVE_LS_A
 
@@ -73,9 +85,15 @@
 /* define if you have h_errlist but not hstrerror */
 #undef HAVE_H_ERRLIST
 
+/* define if you have h_nerr but not hstrerror */
+#undef HAVE_H_NERR
+
 /* define if your system doesn't declare h_errlist */
 #undef HAVE_H_ERRLIST_DECLARATION
 
+/* define if your system doesn't declare h_nerr */
+#undef HAVE_H_NERR_DECLARATION
+
 /* define this if you need a declaration for h_errno */
 #undef HAVE_H_ERRNO_DECLARATION
 
@@ -105,9 +123,15 @@
 #undef HAVE_U_INT32_T
 #undef HAVE_U_INT64_T
 
+/* What is this? */
+#undef HAVE_STRANGE_INT8_T
+
 #define RCSID(msg) \
 static /**/const char *const rcsid[] = { (char *)rcsid, "\100(#)" msg }
 
+#undef VERSION
+#undef PACKAGE
+
 /*
  * Set ORGANIZATION to be the desired organization string printed
  * by the 'kinit' program.  It may have spaces.
@@ -156,6 +180,9 @@ static /**/const char *const rcsid[] = { (char *)rcsid, "\100(#)" msg }
 
 /* telnet stuff ----------------------------------------------- */
 
+/* define this for OTP support */
+#undef OTP
+
 /* define this if you have kerberos 4 */
 #undef KRB4
 
@@ -241,6 +268,10 @@ static /**/const char *const rcsid[] = { (char *)rcsid, "\100(#)" msg }
    type to emit */
 #undef DES_QUAD_DEFAULT
 
+/*
+ * All these are system-specific defines that I would rather not have at all.
+ */
+
 /*
  * AIX braindamage!
  */
@@ -274,3 +305,9 @@ static /**/const char *const rcsid[] = { (char *)rcsid, "\100(#)" msg }
 #if IRIX == 4 && !defined(__STDC__)
 #define __STDC__ 0
 #endif
+
+/* some strange OS/2 stuff.  From <d96-mst@nada.kth.se> */
+
+#ifdef __EMX__
+#define _EMX_TCPIP
+#endif
