@@ -70,7 +70,7 @@
 
 #include <roken.h>
 
-RCSID("$Id: aklog.c,v 1.17 1997/05/11 11:04:25 assar Exp $");
+RCSID("$Id: aklog.c,v 1.18 1997/10/24 10:15:41 assar Exp $");
 
 static int debug = 0;
 
@@ -92,30 +92,28 @@ DEBUG(const char *fmt, ...)
   }
 }
 
-static
-char *
+static char *
 expand_cell_name(char *cell)
 {
-  FILE *F;
+  FILE *f;
   static char buf[128];
   char *p;
 
-  F = fopen(_PATH_CELLSERVDB, "r");
-  if(F == NULL)
+  f = fopen(_PATH_CELLSERVDB, "r");
+  if(f == NULL)
     return cell;
-  do{
-    fgets(buf, 128, F);
-    if(buf[0] == '>'){
-      for(p=buf; *p && *p != ' ' && *p != '\t'; p++);
-      *p=0;
+  while(fgets(buf, sizeof(buf), f) != NULL) {
+    if(buf[0] == '>') {
+      for(p=buf; *p && *p != ' ' && *p != '\t' && *p != '\n'; p++);
+      *p = '\0';
       if(strstr(buf, cell)){
-	fclose(F);
+	fclose(f);
 	return buf + 1;
       }
     }
     buf[0] = 0;
-  }while(!feof(F));
-  fclose(F);
+  }
+  fclose(f);
   return cell;
 }
 
