@@ -39,9 +39,11 @@
 
 #include "bsd_locl.h"
 
-RCSID("$Id: login.c,v 1.94 1996/11/19 22:48:21 assar Exp $");
+RCSID("$Id: login.c,v 1.106 1997/11/09 09:04:25 assar Exp $");
 
+#ifdef OTP
 #include <otp.h>
+#endif
 
 #include "sysv_default.h"
 #ifdef SYSV_SHADOW
@@ -61,15 +63,14 @@ void	login_fbtab (char *, uid_t, gid_t);
 int	klogin (struct passwd *, char *, char *, char *);
 #endif
 
-extern void login (struct utmp *);
-
 #define	TTYGRPNAME	"tty"		/* name of group to own ttys */
 
 /*
- * This bounds the time given to login.  Not a define so it can
- * be patched on machines where it's too small.
+ * This bounds the time given to login.  Change it in
+ * `/etc/default/login'.
  */
-static	u_int	login_timeout = 300;
+
+static	u_int	login_timeout;
 
 #ifdef KERBEROS
 int	notickets = 1;
@@ -90,25 +91,24 @@ static	int	failures;
 static	char	term[64], *hostname, *username, *tty;
 
 static  char rusername[100], lusername[100];
+
 static int
 change_passwd(struct passwd  *who)
 {
-        int             status;
-        int             pid;
-        int             wpid;
+    int status;
+    pid_t pid;
  
-        switch (pid = fork()) {
-	case -1:
-	    perror("Cannot execute /bin/passwd");
-	    sleepexit(1);
-	case 0:
-	    execlp("/bin/passwd", "passwd", who->pw_name, (char *) 0);
-	    _exit(1);
-	default:
-	    while ((wpid = wait(&status)) != -1 && wpid != pid)
-		/* void */ ;
-	    return (status);
-	}
+    switch (pid = fork()) {
+    case -1:
+	warn("fork /bin/passwd");
+	sleepexit(1);
+    case 0:
+	execlp("/bin/passwd", "passwd", who->pw_name, (char *) 0);
+	_exit(1);
+    default:
+	waitpid(pid, &status, 0);
+	return (status);
+    }
 }
 
 #ifndef NO_MOTD /* message of the day stuff */
@@ -171,16 +171,20 @@ main(int argc, char **argv)
 	int rootlogin, rval;
 	int rflag;
 	int changepass = 0;
-	uid_t uid, pwd_uid;
-	char *domain, *p, passwd[128], ep[20], *salt, *ttyn;
+	uid_t uid;
+	char *domain, *p, passwd[128], *ttyn;
 	char tbuf[MaxPathLen + 2], tname[sizeof(_PATH_TTY) + 10];
 	char localhost[MaxHostNameLen];
 	char full_hostname[MaxHostNameLen];
 	int auth_level = AUTH_NONE;
+#ifdef OTP
 	OtpContext otp_ctx;
+#endif
 	int mask = 022;		/* Default umask (set below) */
 	int maxtrys = 5;	/* Default number of allowed failed logins */
 
+	set_progname(argv[0]);
+
 	openlog("login", LOG_ODELAY, LOG_AUTH);
 
         /* Read defaults file and set the login timeout period. */
@@ -220,8 +224,10 @@ main(int argc, char **argv)
 		case 'a':
 			if (strcmp (optarg, "none") == 0)
 				auth_level = AUTH_NONE;
+#ifdef OTP
 			else if (strcmp (optarg, "otp") == 0)
 				auth_level = AUTH_OTP;
+#endif
 			else
 				warnx ("bad value for -a: %s", optarg);
 			break;
@@ -246,21 +252,19 @@ main(int argc, char **argv)
 			break;
 		case 'p':
 			if (getuid()) {
-			    fprintf(stderr,
-                                    "login: -p for super-user only.\n");
-			    exit(1);
+				warnx("-p for super-user only.");
+				exit(1);
                         }
 			pflag = 1;
 			break;
 	        case 'r':
 			if (rflag || hflag) {
-			    printf("Only one of -r and -h allowed\n");
-			    exit(1);
+				warnx("Only one of -r and -h allowed\n");
+				exit(1);
                         }
 			if (getuid()) {
-			    fprintf(stderr,
-                                    "login: -r for super-user only.\n");
-			    exit(1);
+				warnx("-r for super-user only.");
+				exit(1);
                         }
 			rflag = 1;
 			strncpy(full_hostname, optarg, sizeof(full_hostname)-1);
@@ -275,15 +279,18 @@ main(int argc, char **argv)
 			if (!uid)
 				syslog(LOG_ERR, "invalid flag %c", ch);
 			fprintf(stderr,
-				"usage: login [-fp] [-a otp]"
-				"[-h hostname | -r hostname] [username]\n");
+				"usage: login [-fp]"
+#ifdef OTP
+				" [-a otp]"
+#endif
+				" [-h hostname | -r hostname] [username]\n");
 			exit(1);
 		}
 	argc -= optind;
 	argv += optind;
 
 	if (geteuid() != 0) {
-	        fprintf(stderr, "login: only root may use login, use su\n");
+		warnx("only root may use login, use su");
 	        /* Or install login setuid root, which is not necessary */
 		sleep(10);
 		exit(1);
@@ -329,7 +336,7 @@ main(int argc, char **argv)
          */
         ttyn = ttyname(STDIN_FILENO);
         if (ttyn == NULL || *ttyn == '\0') {
-	        sprintf(tname, "%s??", _PATH_TTY);
+	        snprintf(tname, sizeof(tname), "%s??", _PATH_TTY);
 	        ttyn = tname;
 	}
         if ((tty = strchr(ttyn + 1, '/')))
@@ -394,17 +401,26 @@ main(int argc, char **argv)
 
 		setpriority(PRIO_PROCESS, 0, -4);
 
+#ifdef OTP
 		if (otp_challenge (&otp_ctx, username,
 				   ss, sizeof(ss)) == 0)
-			sprintf (prompt, "%s's %s Password: ",
-				 username, ss);
-		else {
+			snprintf (prompt, sizeof(prompt), "%s's %s Password: ",
+				  username, ss);
+		else
+#endif
+		{
 			if (auth_level == AUTH_NONE)
-				sprintf(prompt, "%s's Password: ",
-					username);
+				snprintf(prompt, sizeof(prompt), "%s's Password: ",
+					 username);
 			else {
+				char *s;
+
 				rval = 1;
-				printf ("OTP: %s\n", otp_error(&otp_ctx));
+#ifdef OTP
+				s = otp_error(&otp_ctx);
+				if(s)
+					printf ("OTP: %s\n", s);
+#endif
 				continue;
 			}
 		}
@@ -415,27 +431,39 @@ main(int argc, char **argv)
 
 		/* Verify it somehow */
 
+#ifdef OTP
 		if (otp_verify_user (&otp_ctx, passwd) == 0)
 			rval = 0;
-		else if (pwd == NULL)
+		else
+#endif
+		if (pwd == NULL)
 			;
 		else if (auth_level == AUTH_NONE) {
-			uid_t pwd_uid;
+			uid_t pwd_uid = pwd->pw_uid;
+
+			rval = unix_verify_user (username, passwd);
 
-			pwd_uid = pwd->pw_uid;
-			rval = klogin(pwd, instance, localhost, passwd);
-			if (rval != 0 && rootlogin && pwd_uid != 0)
-				rootlogin = 0;
 			if (rval == 0)
-				authok = 1;
+			  {
+			    if (rootlogin && pwd_uid != 0)
+			      rootlogin = 0;
+			  }
 			else
-				rval = unix_verify_user (username, passwd);
+			  {
+			    rval = klogin(pwd, instance, localhost, passwd);
+			    if (rval != 0 && rootlogin && pwd_uid != 0)
+			      rootlogin = 0;
+			    if (rval == 0)
+			      authok = 1;
+			  }
 		} else {
 			char *s;
 
 			rval = 1;
-			if (s = otp_error(&otp_ctx))
+#ifdef OTP
+			if ((s = otp_error(&otp_ctx)))
 				printf ("OTP: %s\n", s);
+#endif
 		}
 
 		memset (passwd, 0, sizeof(passwd));
@@ -456,9 +484,8 @@ main(int argc, char **argv)
 #endif
 		if (pwd && !rval && rootlogin && !rootterm(tty)
 		    && !rootterm(ttyn)) {
-			fprintf(stderr,
-				"%s login refused on this terminal.\n",
-				pwd->pw_name);
+			warnx("%s login refused on this terminal.",
+			      pwd->pw_name);
 			if (hostname)
 				syslog(LOG_NOTICE,
 				       "LOGIN %s REFUSED FROM %s ON TTY %s",
@@ -499,8 +526,8 @@ main(int argc, char **argv)
 	    const long maxcpu = 46116860184; /* some random constant */
 	    udb = getudbnam(pwd->pw_name);
 	    if(udb == UDB_NULL){
-		fprintf(stderr, "Failed to get UDB entry.\n");
-		exit(1);
+		    warnx("Failed to get UDB entry.");
+		    exit(1);
 	    }
 	    t = udb->ue_pcpulim[UDBRC_INTER];
 	    if(t == 0 || t > maxcpu)
@@ -509,7 +536,7 @@ main(int argc, char **argv)
 		t *= 100 * CLOCKS_PER_SEC;
 
 	    if(limit(C_PROC, 0, L_CPU, t) < 0)
-		perror("login: limit");
+		warn("limit C_PROC");
 
 	    t = udb->ue_jcpulim[UDBRC_INTER];
 	    if(t == 0 || t > maxcpu)
@@ -518,7 +545,7 @@ main(int argc, char **argv)
 		t *= 100 * CLOCKS_PER_SEC;
 
 	    if(limit(C_JOBPROCS, 0, L_CPU, t) < 0)
-		perror("login: limit");
+		warn("limit C_JOBPROCS");
 
 	    nice(udb->ue_nice[UDBRC_INTER]);
 	}
@@ -603,7 +630,7 @@ main(int argc, char **argv)
         sysv_newenv(argc, argv, pwd, term, pflag);
 #ifdef KERBEROS
 	if (krbtkfile_env)
-		setenv("KRBTKFILE", krbtkfile_env, 1);
+	    setenv("KRBTKFILE", krbtkfile_env, 1);
 #endif
 
 	if (tty[sizeof("tty")-1] == 'd')
@@ -649,7 +676,7 @@ main(int argc, char **argv)
 #endif
 		motd();
 		if(!nomailcheck){
-		    sprintf(tbuf, "%s/%s", _PATH_MAILDIR, pwd->pw_name);
+		    snprintf(tbuf, sizeof(tbuf), "%s/%s", _PATH_MAILDIR, pwd->pw_name);
 		    if (stat(tbuf, &st) == 0 && st.st_size != 0)
 			printf("You have %smail.\n",
 			       (st.st_mtime > st.st_atime) ? "new " : "");
@@ -697,10 +724,9 @@ main(int argc, char **argv)
 	{
 	    int uid = rootlogin ? 0 : pwd->pw_uid;
 	    if(setuid(uid) != 0){
-		fprintf(stderr, "Failed to setuid(%d): %s\n", 
-			uid, strerror(errno));
-		if(!rootlogin)
-		    exit(1);
+		    warn("setuid(%d)", uid);
+		    if(!rootlogin)
+			    exit(1);
 	    }
 	}
 		       
@@ -741,8 +767,8 @@ main(int argc, char **argv)
 
 	execlp(pwd->pw_shell, tbuf, 0);
 	if (getuid() == 0) {
-		fprintf(stderr, "Can't exec %s, trying %s\n", 
-			pwd->pw_shell, _PATH_BSHELL);
+		warnx("Can't exec %s, trying %s\n", 
+		      pwd->pw_shell, _PATH_BSHELL);
 		execlp(_PATH_BSHELL, tbuf, 0);
 		err(1, "%s", _PATH_BSHELL);
 	}
@@ -780,8 +806,7 @@ getloginname(int prompt)
 		}
 		if (p > nbuf)
 			if (nbuf[0] == '-')
-				fprintf(stderr,
-				    "login names may not start with '-'.\n");
+				warnx("login names may not start with '-'.");
 			else {
 				*p = '\0';
 				username = nbuf;
@@ -806,7 +831,7 @@ static RETSIGTYPE
 timedout(int signo)
 {
 	fprintf(stderr, "Login timed out after %d seconds\n",
-		      login_timeout);
+		login_timeout);
 	exit(0);
 }
 
@@ -836,7 +861,8 @@ dolastlog(int quiet)
                 if (read(fd, &ll, sizeof(ll)) == sizeof(ll) &&
                     ll.ll_time != 0) {
                         if (pwd->pw_uid && spwd && spwd->sp_inact > 0
-                            && ll.ll_time / DAY + spwd->sp_inact < DAY_NOW) {
+                            && ll.ll_time / (24 * 60 * 60)
+			    + spwd->sp_inact < time(0)) {
                                 printf("Your account has been inactive too long.\n");
                                 sleepexit(1);
                         }
