@@ -38,7 +38,7 @@
 
 #include "kx.h"
 
-RCSID("$Id: common.c,v 1.29 1997/05/25 17:44:38 joda Exp $");
+RCSID("$Id: common.c,v 1.31 1997/11/09 11:13:21 joda Exp $");
 
 char x_socket[MaxPathLen];
 
@@ -88,8 +88,8 @@ copy_encrypted (int fd1, int fd2, des_cblock *iv,
      des_cblock iv1, iv2;
      int num1 = 0, num2 = 0;
 
-     memcpy (&iv1, iv, sizeof(iv1));
-     memcpy (&iv2, iv, sizeof(iv2));
+     memcpy (iv1, *iv, sizeof(iv1));
+     memcpy (iv2, *iv, sizeof(iv2));
      for (;;) {
 	  fd_set fdset;
 	  int ret;
@@ -236,11 +236,13 @@ connect_local_xsocket (unsigned dnr)
 
 int
 create_and_write_cookie (char *xauthfile,
+			 size_t size,
 			 u_char *cookie,
 			 size_t sz)
 {
      Xauth auth;
      char tmp[64];
+     int fd;
      FILE *f;
      char hostname[MaxHostNameLen];
      struct in_addr loopback;
@@ -261,9 +263,16 @@ create_and_write_cookie (char *xauthfile,
      auth.data = (char*)cookie;
      des_rand_data (cookie, sz);
 
-     f = fopen(xauthfile, "w");
-     if (f == NULL)
-          return 1;
+     strncpy(xauthfile, "/tmp/AXXXXX", size);
+     xauthfile[size-1] = 0;
+     fd = mkstemp(xauthfile);
+     if(fd < 0)
+	 return 1;
+     f = fdopen(fd, "r+");
+     if(f == NULL){
+	 close(fd);
+	 return 1;
+     }
      if(XauWriteAuth(f, &auth) == 0) {
 	  fclose(f);
 	  return 1;
