@@ -38,7 +38,7 @@
 
 #include "kx.h"
 
-RCSID("$Id: kxd.c,v 1.38 1997/05/18 20:38:08 assar Exp $");
+RCSID("$Id: kxd.c,v 1.40 1997/11/09 11:14:28 joda Exp $");
 
 /*
  * Signal handler that justs waits for the children when they die.
@@ -303,7 +303,7 @@ doit_conn (int fd, int meta_sock, int flags,
 	return 1;
     }
 
-    sock2 = accept (sock, (struct sockaddr *)&thisaddr, &addrlen);
+    sock2 = accept (sock, (struct sockaddr *)thisaddr, &addrlen);
     if (sock2 < 0) {
 	syslog (LOG_ERR, "accept: %m");
 	return 1;
@@ -364,9 +364,9 @@ doit(int sock, int tcpp)
 	       snprintf (display, display_size, "localhost:%u", display_num);
 	  else
 	       snprintf (display, display_size, ":%u", display_num);
-	  strncpy(xauthfile, tempnam("/tmp", NULL), xauthfile_size);
-	  if(create_and_write_cookie (xauthfile, cookie, cookie_len))
-	       return 1;
+	  if(create_and_write_cookie (xauthfile, xauthfile_size, 
+				      cookie, cookie_len))
+	      return 1;
 
 	  p = msg;
 	  *p++ = ACK;
