@@ -20,10 +20,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
-
-#ifdef SOCKS
-#include <socks.h>
+#include <protos.h>
 #endif
 
 #include <stdlib.h>
@@ -61,12 +58,14 @@ extern struct servent *hes_getservbyname (/* char *, char * */);
 #endif
 #include <errno.h>
 
+#ifdef SOCKS
+#include <socks.h>
+#endif
+
 #include <des.h>
 #include <krb.h>
 
-#if !defined(HAVE_H_ERRNO) || !defined(HAVE_CONFIG_H)
-extern int h_errno;
-#endif
+#include <roken.h>
 
 static int socket_connection ( char *, int );
 static char *getline ( popserver );
@@ -929,7 +928,6 @@ static int
 socket_connection (char *host, int flags)
 {
   struct hostent *hostent;
-  struct servent *servent;
   struct sockaddr_in addr;
   char found_port = 0;
   char *service;
@@ -946,7 +944,11 @@ socket_connection (char *host, int flags)
     {
       hostent = gethostbyname (host);
       try_count++;
-      if ((! hostent) && ((h_errno != TRY_AGAIN) || (try_count == 5)))
+      if ((! hostent)
+#ifdef HAVE_H_ERRNO
+	  && ((h_errno != TRY_AGAIN) || (try_count == 5))
+#endif
+	  )
 	{
 	  strcpy (pop_error, "Could not determine POP server's address");
 	  return (-1);
@@ -961,7 +963,7 @@ socket_connection (char *host, int flags)
 #ifdef HESIOD
   if (! (flags & POP_NO_HESIOD))
     {
-      servent = hes_getservbyname (service, "tcp");
+      struct servent *servent = hes_getservbyname (service, "tcp");
       if (servent)
 	{
 	  addr.sin_port = servent->s_port;
