@@ -55,38 +55,36 @@
 #include <config.h>
 #endif
 
+RCSID("$Id: kerberos.c,v 1.36 1997/11/03 06:12:14 assar Exp $");
+
+#ifdef	KRB4
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#ifdef HAVE_ARPA_TELNET_H
+#include <arpa/telnet.h>
+#endif
+#include <stdio.h>
+#include <des.h>	/* BSD wont include this in krb.h, so we do it here */
+#include <krb.h>
+#include <pwd.h>
+#include <stdlib.h>
+#include <string.h>
 #ifdef SOCKS
 #include <socks.h>
 #endif
 
-RCSID("$Id: kerberos.c,v 1.20 1996/11/19 22:48:34 assar Exp $");
-
-#ifdef	KRB4
-#include <sys/types.h>
-#include <arpa/telnet.h>
-#include <stdio.h>
-#include <des.h>	/* BSD wont include this in krb.h, so we do it here */
-#include <krb.h>
-#ifdef	__STDC__
-#include <stdlib.h>
-#endif
-#ifdef	NO_STRING_H
-#include <strings.h>
-#else
-#include <string.h>
-#endif
+#include <roken.h>
 
 #include "encrypt.h"
 #include "auth.h"
 #include "misc.h"
 
-int kerberos4_cksum P((unsigned char *, int));
+int kerberos4_cksum (unsigned char *, int);
 extern int auth_debug_mode;
 
-static unsigned char str_data[1024] = { IAC, SB, TELOPT_AUTHENTICATION, 0,
+static unsigned char str_data[2048] = { IAC, SB, TELOPT_AUTHENTICATION, 0,
 			  		AUTHTYPE_KERBEROS_V4, };
-static unsigned char str_name[1024] = { IAC, SB, TELOPT_AUTHENTICATION,
-					TELQUAL_NAME, };
 
 #define	KRB_AUTH	0		/* Authentication data follows */
 #define	KRB_REJECT	1		/* Rejected (reason might follow) */
@@ -94,6 +92,10 @@ static unsigned char str_name[1024] = { IAC, SB, TELOPT_AUTHENTICATION,
 #define	KRB_CHALLENGE	3		/* Challenge for mutual auth. */
 #define	KRB_RESPONSE	4		/* Response for mutual auth. */
 
+#define KRB_FORWARD		5	/* */
+#define KRB_FORWARD_ACCEPT	6	/* */
+#define KRB_FORWARD_REJECT	7	/* */
+
 #define KRB_SERVICE_NAME   "rcmd"
 
 static	KTEXT_ST auth;
@@ -102,15 +104,20 @@ static	AUTH_DAT adat;
 static des_cblock session_key;
 static des_key_schedule sched;
 static des_cblock challenge;
+static int auth_done; /* XXX */
+
+static int pack_cred(CREDENTIALS *cred, unsigned char *buf);
+static int unpack_cred(unsigned char *buf, int len, CREDENTIALS *cred);
+
 
 static int
-Data(Authenticator *ap, int type, void *d, int c)
+Data(Authenticator *ap, int type, const void *d, int c)
 {
     unsigned char *p = str_data + 4;
-    unsigned char *cd = (unsigned char *)d;
+    const unsigned char *cd = (const unsigned char *)d;
 
     if (c == -1)
-	c = strlen((char *)cd);
+	c = strlen((const char *)cd);
 
     if (auth_debug_mode) {
 	printf("%s:%d: [%d] (%d)",
@@ -159,8 +166,6 @@ kerberos4_send(char *name, Authenticator *ap)
     KTEXT_ST auth;
     char instance[INST_SZ];
     char *realm;
-    char *krb_realmofhost(const char *);
-    char *krb_get_phost(const char *);
     CREDENTIALS cred;
     int r;
 
@@ -174,7 +179,7 @@ kerberos4_send(char *name, Authenticator *ap)
 
     memset(instance, 0, sizeof(instance));
 
-    if (realm = krb_get_phost(RemoteHostName))
+    if ((realm = krb_get_phost(RemoteHostName)))
 	strncpy(instance, realm, sizeof(instance));
 
     instance[sizeof(instance)-1] = '\0';
@@ -185,11 +190,13 @@ kerberos4_send(char *name, Authenticator *ap)
 	printf("Kerberos V4: no realm for %s\r\n", RemoteHostName);
 	return(0);
     }
-    if (r = krb_mk_req(&auth, KRB_SERVICE_NAME, instance, realm, 0L)) {
+    r = krb_mk_req(&auth, KRB_SERVICE_NAME, instance, realm, 0L);
+    if (r) {
 	printf("mk_req failed: %s\r\n", krb_get_err_text(r));
 	return(0);
     }
-    if (r = krb_get_cred(KRB_SERVICE_NAME, instance, realm, &cred)) {
+    r = krb_get_cred(KRB_SERVICE_NAME, instance, realm, &cred);
+    if (r) {
 	printf("get_cred failed: %s\r\n", krb_get_err_text(r));
 	return(0);
     }
@@ -260,9 +267,11 @@ kerberos4_send_oneway(Authenticator *ap)
 void
 kerberos4_is(Authenticator *ap, unsigned char *data, int cnt)
 {
+    struct sockaddr_in addr;
     char realm[REALM_SZ];
     char instance[INST_SZ];
     int r;
+    int addr_len;
 
     if (cnt-- < 1)
 	return;
@@ -282,9 +291,18 @@ kerberos4_is(Authenticator *ap, unsigned char *data, int cnt)
 	    printd(auth.dat, auth.length);
 	    printf("\r\n");
 	}
-	k_getsockinst(0, instance); /* Telnetd uses socket 0 */
-	if (r = krb_rd_req(&auth, KRB_SERVICE_NAME,
-			   instance, 0, &adat, "")) {
+	k_getsockinst(0, instance, sizeof(instance));
+	addr_len = sizeof(addr);
+	if(getpeername(0, (struct sockaddr *)&addr, &addr_len) < 0) {
+	    if(auth_debug_mode)
+		printf("getpeername failed\r\n");
+	    Data(ap, KRB_REJECT, "getpeername failed", -1);
+	    auth_finished(ap, AUTH_REJECT);
+	    return;
+	}
+	r = krb_rd_req(&auth, KRB_SERVICE_NAME,
+		       instance, addr.sin_addr.s_addr, &adat, "");
+	if (r) {
 	    if (auth_debug_mode)
 		printf("Kerberos failed him as %s\r\n", name);
 	    Data(ap, KRB_REJECT, (void *)krb_get_err_text(r), -1);
@@ -295,24 +313,33 @@ kerberos4_is(Authenticator *ap, unsigned char *data, int cnt)
 	memmove(session_key, adat.session, sizeof(adat.session));
 	krb_kntoln(&adat, name);
 
-	if (UserNameRequested && !kuserok(&adat, UserNameRequested))
+	if (UserNameRequested && !kuserok(&adat, UserNameRequested)){
+	    char ts[MaxPathLen];
+	    struct passwd *pw = getpwnam(UserNameRequested);
+
+	    if(pw){
+		snprintf(ts, sizeof(ts),
+			 "%s%u",
+			 TKT_ROOT,
+			 (unsigned)pw->pw_uid);
+		setenv("KRBTKFILE", ts, 1);
+	    }
 	    Data(ap, KRB_ACCEPT, NULL, 0);
-	else {
-	    char *msg = malloc(ANAME_SZ + 1 + INST_SZ +
-			       REALM_SZ +
-			       strlen(UserNameRequested) + 80);
-	    
+	} else {
+	    char *msg;
+
+	    asprintf (&msg, "user `%s' is not authorized to "
+		      "login as `%s'", 
+		      krb_unparse_name_long(adat.pname, 
+					    adat.pinst, 
+					    adat.prealm), 
+		      UserNameRequested ? UserNameRequested : "<nobody>");
 	    if (msg == NULL)
 		Data(ap, KRB_REJECT, NULL, 0);
-	    sprintf (msg, "user `%s' is not authorized to "
-		     "login as `%s'", 
-		     krb_unparse_name_long(adat.pname, 
-					   adat.pinst, 
-					   adat.prealm), 
-		     UserNameRequested);
-	    
-	    Data(ap, KRB_REJECT, (void *)msg, -1);
-	    free(msg);
+	    else {
+		Data(ap, KRB_REJECT, (void *)msg, -1);
+		free(msg);
+	    }
 	}
 	auth_finished(ap, AUTH_USER);
 	break;
@@ -351,6 +378,51 @@ kerberos4_is(Authenticator *ap, unsigned char *data, int cnt)
 #endif
 	break;
 
+    case KRB_FORWARD:
+	{
+	    des_key_schedule ks;
+	    unsigned char netcred[sizeof(CREDENTIALS)];
+	    CREDENTIALS cred;
+	    int ret;
+	    if(cnt > sizeof(cred))
+		abort();
+
+	    des_set_key(&session_key, ks);
+	    des_pcbc_encrypt((void*)data, (void*)netcred, cnt, 
+			     ks, &session_key, DES_DECRYPT);
+	    unpack_cred(netcred, cnt, &cred);
+	    {
+		if(strcmp(cred.service, KRB_TICKET_GRANTING_TICKET) ||
+		   strncmp(cred.instance, cred.realm, sizeof(cred.instance)) ||
+		   cred.lifetime < 0 || cred.lifetime > 255 ||
+		   cred.kvno < 0 || cred.kvno > 255 ||
+		   cred.issue_date < 0 || 
+		   cred.issue_date > time(0) + CLOCK_SKEW ||
+		   strncmp(cred.pname, adat.pname, sizeof(cred.pname)) ||
+		   strncmp(cred.pinst, adat.pinst, sizeof(cred.pname))){
+		    Data(ap, KRB_FORWARD_REJECT, "Bad credentials", -1);
+		}else{
+		    if((ret = tf_setup(&cred,
+				       cred.pname,
+				       cred.pinst)) == KSUCCESS){
+		        struct passwd *pw = getpwnam(UserNameRequested);
+
+			if (pw)
+			  chown(tkt_string(), pw->pw_uid, pw->pw_gid);
+			Data(ap, KRB_FORWARD_ACCEPT, 0, 0);
+		    } else{
+			Data(ap, KRB_FORWARD_REJECT, 
+			     krb_get_err_text(ret), -1);
+		    }
+		}
+	    }
+	    memset(data, 0, cnt);
+	    memset(ks, 0, sizeof(ks));
+	    memset(&cred, 0, sizeof(cred));
+	}
+	
+	break;
+
     default:
 	if (auth_debug_mode)
 	    printf("Unknown Kerberos option %d\r\n", data[-1]);
@@ -368,27 +440,32 @@ kerberos4_reply(Authenticator *ap, unsigned char *data, int cnt)
 	return;
     switch (*data++) {
     case KRB_REJECT:
-	if (cnt > 0) {
-	    printf("[ Kerberos V4 refuses authentication because %.*s ]\r\n",
-		   cnt, data);
-	} else
-	    printf("[ Kerberos V4 refuses authentication ]\r\n");
-	auth_send_retry();
+	if(auth_done){ /* XXX Ick! */
+	    printf("[ Kerberos V4 received unknown opcode ]\r\n");
+	}else{
+	    printf("[ Kerberos V4 refuses authentication ");
+	    if (cnt > 0) 
+		printf("because %.*s ", cnt, data);
+	    printf("]\r\n");
+	    auth_send_retry();
+	}
 	return;
     case KRB_ACCEPT:
 	printf("[ Kerberos V4 accepts you ]\r\n");
+	auth_done = 1;
 	if ((ap->way & AUTH_HOW_MASK) == AUTH_HOW_MUTUAL) {
 	    /*
 	     * Send over the encrypted challenge.
 	     */
 	    Data(ap, KRB_CHALLENGE, session_key, 
 		 sizeof(session_key));
-#ifdef ENCRYPTION
 	    des_ecb_encrypt(&session_key, &session_key, sched, 1);
 	    skey.type = SK_DES;
 	    skey.length = 8;
 	    skey.data = session_key;
 	    encrypt_session_key(&skey, 0);
+#if 0
+	    kerberos4_forward(ap);
 #endif
 	    return;
 	}
@@ -405,6 +482,13 @@ kerberos4_reply(Authenticator *ap, unsigned char *data, int cnt)
 	printf("[ Kerberos V4 challenge successful ]\r\n");
 	auth_finished(ap, AUTH_USER);
 	break;
+    case KRB_FORWARD_ACCEPT:
+	printf("[ Kerberos V4 accepted forwarded credentials ]\r\n");
+	break;
+    case KRB_FORWARD_REJECT:
+	printf("[ Kerberos V4 rejected forwarded credentials: `%.*s']\r\n",
+	       cnt, data);
+	break;
     default:
 	if (auth_debug_mode)
 	    printf("Unknown Kerberos option %d\r\n", data[-1]);
@@ -468,12 +552,12 @@ kerberos4_printsub(unsigned char *data, int cnt, unsigned char *buf, int buflen)
 	goto common2;
 
     default:
-	sprintf(lbuf, " %d (unknown)", data[3]);
+	snprintf(lbuf, sizeof(lbuf), " %d (unknown)", data[3]);
 	strncpy((char *)buf, lbuf, buflen);
     common2:
 	BUMP(buf, buflen);
 	for (i = 4; i < cnt; i++) {
-	    sprintf(lbuf, " %d", data[i]);
+	    snprintf(lbuf, sizeof(lbuf), " %d", data[i]);
 	    strncpy((char *)buf, lbuf, buflen);
 	    BUMP(buf, buflen);
 	}
@@ -515,18 +599,73 @@ kerberos4_cksum(unsigned char *d, int n)
 	}
     return(ck);
 }
-#endif
 
-#ifdef notdef
-
-prkey(msg, key)
-     char *msg;
-     unsigned char *key;
+static int
+pack_cred(CREDENTIALS *cred, unsigned char *buf)
 {
-    int i;
-    printf("%s:", msg);
-    for (i = 0; i < 8; i++)
-	printf(" %3d", key[i]);
-    printf("\r\n");
+    unsigned char *p = buf;
+    
+    p += krb_put_nir(cred->service, cred->instance, cred->realm, p);
+    memcpy(p, cred->session, 8);
+    p += 8;
+    *p++ = cred->lifetime;
+    *p++ = cred->kvno;
+    p += krb_put_int(cred->ticket_st.length, p, 4);
+    memcpy(p, cred->ticket_st.dat, cred->ticket_st.length);
+    p += cred->ticket_st.length;
+    p += krb_put_int(cred->issue_date, p, 4);
+    p += krb_put_nir(cred->pname, cred->pinst, NULL, p);
+    return p - buf;
 }
-#endif
+
+static int
+unpack_cred(unsigned char *buf, int len, CREDENTIALS *cred)
+{
+    unsigned char *p = buf;
+
+    p += krb_get_nir(p, cred->service, cred->instance, cred->realm);
+    memcpy(cred->session, p, 8);
+    p += 8;
+    cred->lifetime = *p++;
+    cred->kvno = *p++;
+    p += krb_get_int(p, &cred->ticket_st.length, 4, 0);
+    memcpy(cred->ticket_st.dat, p, cred->ticket_st.length);
+    cred->ticket_st.mbz = 0;
+    p += krb_get_int(p, (u_int32_t *)&cred->issue_date, 4, 0);
+    p += krb_get_nir(p, cred->pname, cred->pinst, NULL);
+    return 0;
+}
+
+
+int
+kerberos4_forward(Authenticator *ap)
+{
+    CREDENTIALS cred;
+    char *realm;
+    des_key_schedule ks;
+    int len;
+    unsigned char netcred[sizeof(CREDENTIALS)];
+    int ret;
+
+    realm = krb_realmofhost(RemoteHostName);
+    if(realm == NULL)
+	return -1;
+    memset(&cred, 0, sizeof(cred));
+    ret = krb_get_cred(KRB_TICKET_GRANTING_TICKET,
+		       realm,
+		       realm, 
+		       &cred);
+    if(ret)
+	return ret;
+    des_set_key(&session_key, ks);
+    len = pack_cred(&cred, netcred);
+    des_pcbc_encrypt((void*)netcred, (void*)netcred, len,
+		     ks, &session_key, DES_ENCRYPT);
+    memset(ks, 0, sizeof(ks));
+    Data(ap, KRB_FORWARD, netcred, len);
+    memset(netcred, 0, sizeof(netcred));
+    return 0;
+}
+
+#endif /* KRB4 */
+
