@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1995, 1996 Kungliga Tekniska Hgskolan (Royal Institute
- * of Technology, Stockholm, Sweden).
+ * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -36,7 +36,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: krb_locl.h,v 1.31 1996/11/17 11:07:22 joda Exp $ */
+/* $Id: krb_locl.h,v 1.44 1997/10/28 15:37:40 bg Exp $ */
 
 #ifndef __krb_locl_h
 #define __krb_locl_h
@@ -45,10 +45,6 @@
 #include <config.h>
 #endif
 
-#ifdef SOCKS
-#include <socks.h>
-#endif
-
 #include "protos.h"
 
 #include <stdio.h>
@@ -77,13 +73,11 @@
 #ifdef TIME_WITH_SYS_TIME
 #include <sys/time.h>
 #include <time.h>
-#else
-#ifdef HAVE_SYS_TIME_H
+#elif defined(HAVE_SYS_TIME_H)
 #include <sys/time.h>
 #else
 #include <time.h>
-#endif /* !HAVE_SYS_TIME_H */
-#endif /* !TIME_WITH_SYS_TIME */
+#endif
 
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
@@ -123,21 +117,27 @@
 #include <resolv.h>
 #endif
 
+#ifdef SOCKS
+#include <socks.h>
+#endif
+
 #include <roken.h>
 
 #include <krb.h>
 #include <prot.h>
 
 #include "resolve.h"
+#include "krb_log.h"
 
 /* --- */
 
 /* Globals! */
 extern int krb_debug;
 extern int krb_ap_req_debug;
+extern int krb_dns_debug;
 
 /* Utils */
-int krb_name_to_name(const char *, char *);
+int krb_name_to_name(const char *, char *, size_t);
 
 void encrypt_ktext(KTEXT cip, des_cblock *key, int encrypt);
 int kdc_reply_cred(KTEXT cip, CREDENTIALS *cred);
@@ -147,4 +147,19 @@ int kdc_reply_cipher(KTEXT reply, KTEXT cip);
 int gettimeofday (struct timeval *, void *);
 #endif
 
+void k_ricercar(char*);
+
+/* safe multiple strcat */
+int k_concat(char*, size_t, ...);
+int k_vconcat(char*, size_t, va_list);
+
+/* mallocing versions of the above */
+size_t k_vmconcat (char**, size_t, va_list); 
+size_t k_mconcat (char**, size_t, ...);
+
+/* used in rd_safe.c and mk_safe.c */
+
+void fixup_quad_cksum(void *start, size_t len, des_cblock *key, 
+		      void *new_checksum, void *old_checksum, int little);
+
 #endif /*  __krb_locl_h */
