@@ -1,8 +1,3 @@
-/*
- * This software may now be redistributed outside the US.
- *
- */
-
 /* 
   Copyright (C) 1989 by the Massachusetts Institute of Technology
 
@@ -26,7 +21,7 @@ or implied warranty.
 
 #include "krb_locl.h"
 
-RCSID("$Id: sendauth.c,v 1.13 1996/11/14 16:53:01 joda Exp $");
+RCSID("$Id: sendauth.c,v 1.15 1997/04/18 14:11:36 joda Exp $");
 
 /*
  * krb_sendauth() transmits a ticket over a file descriptor for a
@@ -159,8 +154,8 @@ krb_sendauth(int32_t options,	/* bit-pattern of options */
 
 	des_key_sched(&cred->session, schedule);
 
-	ret = krb_check_auth (&buf, checksum, msg_data, &cred->session, schedule,
-			      laddr, faddr);
+	ret = krb_check_auth (&buf, checksum, msg_data, &cred->session, 
+			      schedule, laddr, faddr);
 	if (ret != KSUCCESS)
 	    return ret;
     }
