@@ -1,8 +1,3 @@
-/*
- * This software may now be redistributed outside the US.
- *
- */
-
 /* 
   Copyright (C) 1989 by the Massachusetts Institute of Technology
 
@@ -23,13 +18,10 @@ this software for any purpose.  It is provided "as is" without express
 or implied warranty.
 
   */
-/* These functions are not needed in Windows95/NT implementation,
- * since no ticket files are used.
- */
         
 #include "krb_locl.h"
 
-RCSID("$Id: tf_util.c,v 1.18 1996/11/19 22:48:49 assar Exp $");
+RCSID("$Id: tf_util.c,v 1.25 1997/11/04 09:44:28 bg Exp $");
 
 
 #define TOO_BIG -1
@@ -38,6 +30,9 @@ RCSID("$Id: tf_util.c,v 1.18 1996/11/19 22:48:49 assar Exp $");
 					 * locked */
 #define	TF_LCK_RETRY_COUNT	(50)	/* number of retries	*/
 
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
 
 /*
  * fd must be initialized to something that won't ever occur as a real
@@ -129,97 +124,99 @@ static int tf_read(void *s, int n);
 int
 tf_init(char *tf_name, int rw)
 {
-	/* Unix implementation */
-    int     wflag;
-    struct stat stat_buf;
-    int		i_retry;
+  /* Unix implementation */
+  int wflag;
+  struct stat stat_buf;
+  int i_retry;
 
-
-    switch (rw) {
-    case R_TKT_FIL:
-	wflag = 0;
-	break;
-    case W_TKT_FIL:
-	wflag = 1;
-	break;
+  switch (rw) {
+  case R_TKT_FIL:
+    wflag = 0;
+    break;
+  case W_TKT_FIL:
+    wflag = 1;
+    break;
+  default:
+    if (krb_debug)
+      krb_warning("tf_init: illegal parameter\n");
+    return TKT_FIL_ACC;
+  }
+  if (lstat(tf_name, &stat_buf) < 0)
+    switch (errno) {
+    case ENOENT:
+      return NO_TKT_FIL;
     default:
-	if (krb_debug)
-	  krb_warning("tf_init: illegal parameter\n");
-	return TKT_FIL_ACC;
-    }
-    if (lstat(tf_name, &stat_buf) < 0)
-	switch (errno) {
-	case ENOENT:
-	    return NO_TKT_FIL;
-	default:
-	    return TKT_FIL_ACC;
-	}
-    /* The old code tried to guess when the calling program was
-     * running set-uid, this is now removed - the kerberos library
-     * does not (or shouldn't) know anything about user-ids.
-
-     * All library functions now assume that the right userids are set
-     * upon entry, therefore there is no need to test permissions like
-     * before. If the file is openable, just open it.
-     */
-
-    if(!S_ISREG(stat_buf.st_mode))
       return TKT_FIL_ACC;
+    }
+  if (!S_ISREG(stat_buf.st_mode))
+    return TKT_FIL_ACC;
 
+  /* The code tries to guess when the calling program is running
+   * set-uid and prevent unauthorized access.
+   *
+   * All library functions now assume that the right set of userids
+   * are set upon entry, therefore it's not strictly necessary to
+   * perform these test for programs adhering to these assumptions.
+   */
+  {
+    uid_t me = getuid();
+    if (stat_buf.st_uid != me && me != 0)
+      return TKT_FIL_ACC;
+  }
 
-    /*
-     * If "wflag" is set, open the ticket file in append-writeonly mode
-     * and lock the ticket file in exclusive mode.  If unable to lock
-     * the file, sleep and try again.  If we fail again, return with the
-     * proper error message. 
-     */
+  /*
+   * If "wflag" is set, open the ticket file in append-writeonly mode
+   * and lock the ticket file in exclusive mode.  If unable to lock
+   * the file, sleep and try again.  If we fail again, return with the
+   * proper error message. 
+   */
 
-    curpos = sizeof(tfbfr);
+  curpos = sizeof(tfbfr);
 
     
-    if (wflag) {
-      fd = open(tf_name, O_RDWR, 0600);
-      if (fd < 0) {
-	return TKT_FIL_ACC;
-      }
-      for (i_retry = 0; i_retry < TF_LCK_RETRY_COUNT; i_retry++) {
-	if (k_flock(fd, K_LOCK_EX | K_LOCK_NB) < 0) {
-	  if (krb_debug)
-	    krb_warning("tf_init: retry %d of write lock of `%s'.\n",
-			i_retry, tf_name);
-	  sleep (TF_LCK_RETRY);
-	} else {
-	  return KSUCCESS;		/* all done */
-	}
-      }
-      close (fd);
-      fd = -1;
-      return TKT_FIL_LCK;
-    }
-    /*
-     * Otherwise "wflag" is not set and the ticket file should be opened
-     * for read-only operations and locked for shared access. 
-     */
-
-    fd = open(tf_name, O_RDONLY, 0600);
+  if (wflag) {
+    fd = open(tf_name, O_RDWR | O_BINARY, 0600);
     if (fd < 0) {
-	return TKT_FIL_ACC;
+      return TKT_FIL_ACC;
     }
-
     for (i_retry = 0; i_retry < TF_LCK_RETRY_COUNT; i_retry++) {
-	if (k_flock(fd, K_LOCK_SH | K_LOCK_NB) < 0) {
-	  if (krb_debug)
-	    krb_warning("tf_init: retry %d of read lock of `%s'.\n",
-			i_retry, tf_name);
-	  sleep (TF_LCK_RETRY);
-	} else {
-	  return KSUCCESS;		/* all done */
-	}
+      if (k_flock(fd, K_LOCK_EX | K_LOCK_NB) < 0) {
+	if (krb_debug)
+	  krb_warning("tf_init: retry %d of write lock of `%s'.\n",
+		      i_retry, tf_name);
+	sleep (TF_LCK_RETRY);
+      } else {
+	return KSUCCESS;		/* all done */
       }
-    /* failure */
-    close(fd);
+    }
+    close (fd);
     fd = -1;
     return TKT_FIL_LCK;
+  }
+  /*
+   * Otherwise "wflag" is not set and the ticket file should be opened
+   * for read-only operations and locked for shared access. 
+   */
+
+  fd = open(tf_name, O_RDONLY | O_BINARY, 0600);
+  if (fd < 0) {
+    return TKT_FIL_ACC;
+  }
+
+  for (i_retry = 0; i_retry < TF_LCK_RETRY_COUNT; i_retry++) {
+    if (k_flock(fd, K_LOCK_SH | K_LOCK_NB) < 0) {
+      if (krb_debug)
+	krb_warning("tf_init: retry %d of read lock of `%s'.\n",
+		    i_retry, tf_name);
+      sleep (TF_LCK_RETRY);
+    } else {
+      return KSUCCESS;		/* all done */
+    }
+  }
+  /* failure */
+  close(fd);
+  fd = -1;
+  return TKT_FIL_LCK;
 }
 
 /*
@@ -238,35 +235,35 @@ tf_init(char *tf_name, int rw)
 int
 tf_create(char *tf_name)
 {
-    struct stat statbuf;
-    char garbage[BUFSIZ];
+  struct stat statbuf;
+  char garbage[BUFSIZ];
 
-    fd = open(tf_name, O_RDWR, 0);
-    if (fd >= 0) {
-	if (fstat (fd, &statbuf) == 0) {
-	    int i;
+  fd = open(tf_name, O_RDWR | O_BINARY, 0);
+  if (fd >= 0) {
+    if (fstat (fd, &statbuf) == 0) {
+      int i;
 
-	    for (i = 0; i < statbuf.st_size; i += sizeof(garbage))
-		write (fd, garbage, sizeof(garbage));
-	}
-	close (fd);
+      for (i = 0; i < statbuf.st_size; i += sizeof(garbage))
+	write (fd, garbage, sizeof(garbage));
     }
+    close (fd);
+  }
 
-    if (unlink (tf_name) && errno != ENOENT)
-	return TKT_FIL_ACC;
+  if (unlink (tf_name) && errno != ENOENT)
+    return TKT_FIL_ACC;
 
-    fd = open(tf_name, O_RDWR | O_CREAT | O_EXCL, 0600);
-    if (fd < 0)
-	return TKT_FIL_ACC;
+  fd = open(tf_name, O_RDWR | O_CREAT | O_EXCL | O_BINARY, 0600);
+  if (fd < 0)
+    return TKT_FIL_ACC;
+  if (k_flock(fd, K_LOCK_EX | K_LOCK_NB) < 0) {
+    sleep(TF_LCK_RETRY);
     if (k_flock(fd, K_LOCK_EX | K_LOCK_NB) < 0) {
-	sleep(TF_LCK_RETRY);
-	if (k_flock(fd, K_LOCK_EX | K_LOCK_NB) < 0) {
-	    close(fd);
-	    fd = -1;
-	    return TKT_FIL_LCK;
-	}
+      close(fd);
+      fd = -1;
+      return TKT_FIL_LCK;
     }
-    return KSUCCESS;
+  }
+  return KSUCCESS;
 }
 
 /*
@@ -281,18 +278,18 @@ tf_create(char *tf_name)
 int
 tf_get_pname(char *p)
 {
-    if (fd < 0) {
-	if (krb_debug)
-	    krb_warning("tf_get_pname called before tf_init.\n");
-	return TKT_FIL_INI;
+  if (fd < 0) {
+    if (krb_debug)
+      krb_warning("tf_get_pname called before tf_init.\n");
+    return TKT_FIL_INI;
+  }
+  if (tf_gets(p, ANAME_SZ) < 2)	/* can't be just a null */
+    {
+      if (krb_debug) 
+	krb_warning ("tf_get_pname: pname < 2.\n");
+      return TKT_FIL_FMT;
     }
-    if (tf_gets(p, ANAME_SZ) < 2)	/* can't be just a null */
-      {
-	if (krb_debug) 
-	  krb_warning ("tf_get_pname: pname < 2.\n");
-	return TKT_FIL_FMT;
-      }
-    return KSUCCESS;
+  return KSUCCESS;
 }
 
 /*
@@ -303,17 +300,17 @@ tf_get_pname(char *p)
 int
 tf_put_pname(char *p)
 {
-	unsigned count;
+  unsigned count;
 
-    if (fd < 0) {
-	if (krb_debug)
-	    krb_warning("tf_put_pname called before tf_create.\n");
-	return TKT_FIL_INI;
-    }
-    count = strlen(p)+1;
-    if (write(fd,p,count) != count)
-        return(KFAILURE);
-    return KSUCCESS;
+  if (fd < 0) {
+    if (krb_debug)
+      krb_warning("tf_put_pname called before tf_create.\n");
+    return TKT_FIL_INI;
+  }
+  count = strlen(p)+1;
+  if (write(fd,p,count) != count)
+    return(KFAILURE);
+  return KSUCCESS;
 }
 
 /*
@@ -329,18 +326,18 @@ tf_put_pname(char *p)
 int
 tf_get_pinst(char *inst)
 {
-    if (fd < 0) {
-	if (krb_debug)
-	    krb_warning("tf_get_pinst called before tf_init.\n");
-	return TKT_FIL_INI;
+  if (fd < 0) {
+    if (krb_debug)
+      krb_warning("tf_get_pinst called before tf_init.\n");
+    return TKT_FIL_INI;
+  }
+  if (tf_gets(inst, INST_SZ) < 1)
+    {
+      if (krb_debug)
+	krb_warning("tf_get_pinst: inst_sz < 1.\n");
+      return TKT_FIL_FMT;
     }
-    if (tf_gets(inst, INST_SZ) < 1)
-      {
-	if (krb_debug)
-	  krb_warning("tf_get_pinst: inst_sz < 1.\n");
-	return TKT_FIL_FMT;
-      }
-    return KSUCCESS;
+  return KSUCCESS;
 }
 
 /*
@@ -351,17 +348,17 @@ tf_get_pinst(char *inst)
 int
 tf_put_pinst(char *inst)
 {
-	unsigned count;
+  unsigned count;
 
-    if (fd < 0) {
-	if (krb_debug)
-	    krb_warning("tf_put_pinst called before tf_create.\n");
-	return TKT_FIL_INI;
-    }
-    count = strlen(inst)+1;
-    if (write(fd,inst,count) != count)
-        return(KFAILURE);
-	return KSUCCESS;
+  if (fd < 0) {
+    if (krb_debug)
+      krb_warning("tf_put_pinst called before tf_create.\n");
+    return TKT_FIL_INI;
+  }
+  count = strlen(inst)+1;
+  if (write(fd,inst,count) != count)
+    return(KFAILURE);
+  return KSUCCESS;
 }
 
 /*
@@ -378,66 +375,66 @@ tf_put_pinst(char *inst)
 int
 tf_get_cred(CREDENTIALS *c)
 {
-    KTEXT   ticket = &c->ticket_st;	/* pointer to ticket */
-    int     k_errno;
+  KTEXT   ticket = &c->ticket_st;	/* pointer to ticket */
+  int     k_errno;
 
-    if (fd < 0) {
-	if (krb_debug)
-	    krb_warning ("tf_get_cred called before tf_init.\n");
-	return TKT_FIL_INI;
+  if (fd < 0) {
+    if (krb_debug)
+      krb_warning ("tf_get_cred called before tf_init.\n");
+    return TKT_FIL_INI;
+  }
+  if ((k_errno = tf_gets(c->service, SNAME_SZ)) < 2)
+    switch (k_errno) {
+    case TOO_BIG:
+      if (krb_debug)
+	krb_warning("tf_get_cred: too big service cred.\n");
+    case 1:		/* can't be just a null */
+      tf_close();
+      if (krb_debug)
+	krb_warning("tf_get_cred: null service cred.\n");
+      return TKT_FIL_FMT;
+    case 0:
+      return EOF;
     }
-    if ((k_errno = tf_gets(c->service, SNAME_SZ)) < 2)
-	switch (k_errno) {
-	case TOO_BIG:
-	  if (krb_debug)
-	    krb_warning("tf_get_cred: too big service cred.\n");
-	case 1:		/* can't be just a null */
-	  tf_close();
-	  if (krb_debug)
-	    krb_warning("tf_get_cred: null service cred.\n");
-	  return TKT_FIL_FMT;
-	case 0:
-	  return EOF;
-	}
-    if ((k_errno = tf_gets(c->instance, INST_SZ)) < 1)
-	switch (k_errno) {
-	case TOO_BIG:
-	  if (krb_debug)
-	    krb_warning ("tf_get_cred: too big instance cred.\n");
-	  return TKT_FIL_FMT;
-	case 0:
-	  return EOF;
-	}
-    if ((k_errno = tf_gets(c->realm, REALM_SZ)) < 2)
-	switch (k_errno) {
-	case TOO_BIG:
-	  if (krb_debug)
-	    krb_warning ("tf_get_cred: too big realm cred.\n");
-	case 1:		/* can't be just a null */
-	  tf_close();
-	  if (krb_debug)
-	    krb_warning ("tf_get_cred: null realm cred.\n");
-	  return TKT_FIL_FMT;
-	case 0:
-	  return EOF;
-	}
-    if (
-	tf_read((c->session), DES_KEY_SZ) < 1 ||
-	tf_read(&(c->lifetime), sizeof(c->lifetime)) < 1 ||
-	tf_read(&(c->kvno), sizeof(c->kvno)) < 1 ||
-	tf_read(&(ticket->length), sizeof(ticket->length))
-	< 1 ||
-    /* don't try to read a silly amount into ticket->dat */
-	ticket->length > MAX_KTXT_LEN ||
-	tf_read((ticket->dat), ticket->length) < 1 ||
-	tf_read(&(c->issue_date), sizeof(c->issue_date)) < 1
-	) {
-	tf_close();
-	if (krb_debug)
-	  krb_warning ("tf_get_cred: failed tf_read.\n");
-	return TKT_FIL_FMT;
+  if ((k_errno = tf_gets(c->instance, INST_SZ)) < 1)
+    switch (k_errno) {
+    case TOO_BIG:
+      if (krb_debug)
+	krb_warning ("tf_get_cred: too big instance cred.\n");
+      return TKT_FIL_FMT;
+    case 0:
+      return EOF;
     }
-    return KSUCCESS;
+  if ((k_errno = tf_gets(c->realm, REALM_SZ)) < 2)
+    switch (k_errno) {
+    case TOO_BIG:
+      if (krb_debug)
+	krb_warning ("tf_get_cred: too big realm cred.\n");
+    case 1:		/* can't be just a null */
+      tf_close();
+      if (krb_debug)
+	krb_warning ("tf_get_cred: null realm cred.\n");
+      return TKT_FIL_FMT;
+    case 0:
+      return EOF;
+    }
+  if (
+      tf_read((c->session), DES_KEY_SZ) < 1 ||
+      tf_read(&(c->lifetime), sizeof(c->lifetime)) < 1 ||
+      tf_read(&(c->kvno), sizeof(c->kvno)) < 1 ||
+      tf_read(&(ticket->length), sizeof(ticket->length))
+      < 1 ||
+      /* don't try to read a silly amount into ticket->dat */
+      ticket->length > MAX_KTXT_LEN ||
+      tf_read((ticket->dat), ticket->length) < 1 ||
+      tf_read(&(c->issue_date), sizeof(c->issue_date)) < 1
+      ) {
+    tf_close();
+    if (krb_debug)
+      krb_warning ("tf_get_cred: failed tf_read.\n");
+    return TKT_FIL_FMT;
+  }
+  return KSUCCESS;
 }
 
 /*
@@ -451,12 +448,12 @@ tf_get_cred(CREDENTIALS *c)
 void
 tf_close(void)
 {
-    if (!(fd < 0)) {
-	k_flock(fd, K_LOCK_UN);
-	close(fd);
-	fd = -1;		/* see declaration of fd above */
-    }
-    memset(tfbfr, 0, sizeof(tfbfr));
+  if (!(fd < 0)) {
+    k_flock(fd, K_LOCK_UN);
+    close(fd);
+    fd = -1;		/* see declaration of fd above */
+  }
+  memset(tfbfr, 0, sizeof(tfbfr));
 }
 
 /*
@@ -480,28 +477,28 @@ tf_close(void)
 static int
 tf_gets(char *s, int n)
 {
-    int count;
+  int count;
 
-    if (fd < 0) {
-	if (krb_debug)
-	    krb_warning ("tf_gets called before tf_init.\n");
-	return TKT_FIL_INI;
+  if (fd < 0) {
+    if (krb_debug)
+      krb_warning ("tf_gets called before tf_init.\n");
+    return TKT_FIL_INI;
+  }
+  for (count = n - 1; count > 0; --count) {
+    if (curpos >= sizeof(tfbfr)) {
+      lastpos = read(fd, tfbfr, sizeof(tfbfr));
+      curpos = 0;
     }
-    for (count = n - 1; count > 0; --count) {
-	if (curpos >= sizeof(tfbfr)) {
-	    lastpos = read(fd, tfbfr, sizeof(tfbfr));
-	    curpos = 0;
-	}
-	if (curpos == lastpos) {
-	    tf_close();
-	    return 0;
-	}
-	*s = tfbfr[curpos++];
-	if (*s++ == '\0')
-	    return (n - count);
+    if (curpos == lastpos) {
+      tf_close();
+      return 0;
     }
-    tf_close();
-    return TOO_BIG;
+    *s = tfbfr[curpos++];
+    if (*s++ == '\0')
+      return (n - count);
+  }
+  tf_close();
+  return TOO_BIG;
 }
 
 /*
@@ -520,21 +517,21 @@ tf_gets(char *s, int n)
 static int
 tf_read(void *v, int n)
 {
-    char *s = (char *)v;
-    int count;
+  char *s = (char *)v;
+  int count;
     
-    for (count = n; count > 0; --count) {
-	if (curpos >= sizeof(tfbfr)) {
-	    lastpos = read(fd, tfbfr, sizeof(tfbfr));
-	    curpos = 0;
-	}
-	if (curpos == lastpos) {
-	    tf_close();
-	    return 0;
-	}
-	*s++ = tfbfr[curpos++];
+  for (count = n; count > 0; --count) {
+    if (curpos >= sizeof(tfbfr)) {
+      lastpos = read(fd, tfbfr, sizeof(tfbfr));
+      curpos = 0;
     }
-    return n;
+    if (curpos == lastpos) {
+      tf_close();
+      return 0;
+    }
+    *s++ = tfbfr[curpos++];
+  }
+  return n;
 }
      
 /*
@@ -561,53 +558,91 @@ tf_save_cred(char *service,	/* Service name */
 	     KTEXT ticket,	/* The ticket itself */
 	     u_int32_t issue_date) /* The issue time */
 {
-    int     count;		/* count for write */
+  int count;			/* count for write */
 
-    if (fd < 0) {		/* fd is ticket file as set by tf_init */
-	  if (krb_debug)
-	      krb_warning ("tf_save_cred called before tf_init.\n");
-	  return TKT_FIL_INI;
-    }
-    /* Find the end of the ticket file */
-    lseek(fd, 0L, SEEK_END);
+  if (fd < 0) {			/* fd is ticket file as set by tf_init */
+    if (krb_debug)
+      krb_warning ("tf_save_cred called before tf_init.\n");
+    return TKT_FIL_INI;
+  }
+  /* Find the end of the ticket file */
+  lseek(fd, 0L, SEEK_END);
 
-    /* Write the ticket and associated data */
-    /* Service */
-    count = strlen(service) + 1;
-    if (write(fd, service, count) != count)
-	goto bad;
-    /* Instance */
-    count = strlen(instance) + 1;
-    if (write(fd, instance, count) != count)
-	goto bad;
-    /* Realm */
-    count = strlen(realm) + 1;
-    if (write(fd, realm, count) != count)
-	goto bad;
-    /* Session key */
-    if (write(fd, session, 8) != 8)
-	goto bad;
-    /* Lifetime */
-    if (write(fd, &lifetime, sizeof(int)) != sizeof(int))
-	goto bad;
-    /* Key vno */
-    if (write(fd, &kvno, sizeof(int)) != sizeof(int))
-	goto bad;
-    /* Tkt length */
-    if (write(fd, &(ticket->length), sizeof(int)) !=
-	sizeof(int))
-	goto bad;
-    /* Ticket */
-    count = ticket->length;
-    if (write(fd, ticket->dat, count) != count)
-	goto bad;
-    /* Issue date */
-    if (write(fd, &issue_date, sizeof(issue_date)) != sizeof(issue_date))
-	goto bad;
+  /* Write the ticket and associated data */
+  /* Service */
+  count = strlen(service) + 1;
+  if (write(fd, service, count) != count)
+    goto bad;
+  /* Instance */
+  count = strlen(instance) + 1;
+  if (write(fd, instance, count) != count)
+    goto bad;
+  /* Realm */
+  count = strlen(realm) + 1;
+  if (write(fd, realm, count) != count)
+    goto bad;
+  /* Session key */
+  if (write(fd, session, 8) != 8)
+    goto bad;
+  /* Lifetime */
+  if (write(fd, &lifetime, sizeof(int)) != sizeof(int))
+    goto bad;
+  /* Key vno */
+  if (write(fd, &kvno, sizeof(int)) != sizeof(int))
+    goto bad;
+  /* Tkt length */
+  if (write(fd, &(ticket->length), sizeof(int)) !=
+      sizeof(int))
+    goto bad;
+  /* Ticket */
+  count = ticket->length;
+  if (write(fd, ticket->dat, count) != count)
+    goto bad;
+  /* Issue date */
+  if (write(fd, &issue_date, sizeof(issue_date)) != sizeof(issue_date))
+    goto bad;
 
-    /* Actually, we should check each write for success */
-    return (KSUCCESS);
+  return (KSUCCESS);
 bad:
-    return (KFAILURE);
+  return (KFAILURE);
 }
 	  
+int
+tf_setup(CREDENTIALS *cred, char *pname, char *pinst)
+{
+    int ret;
+    ret = tf_create(tkt_string());
+    if (ret != KSUCCESS)
+	return ret;
+
+    if (tf_put_pname(pname) != KSUCCESS ||
+	tf_put_pinst(pinst) != KSUCCESS) {
+	tf_close();
+	return INTK_ERR;
+    }
+
+    ret = tf_save_cred(cred->service, cred->instance, cred->realm, 
+		       cred->session, cred->lifetime, cred->kvno,
+		       &cred->ticket_st, cred->issue_date);
+    tf_close();
+    return ret;
+}
+
+int
+in_tkt(char *pname, char *pinst)
+{
+  int ret;
+  
+  ret = tf_create (tkt_string());
+  if (ret != KSUCCESS)
+    return ret;
+
+    if (tf_put_pname(pname) != KSUCCESS ||
+	tf_put_pinst(pinst) != KSUCCESS) {
+	tf_close();
+	return INTK_ERR;
+    }
+
+    tf_close();
+    return KSUCCESS;
+}
